@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2012-2014 CoNWeT Lab., Universidad Politécnica de Madrid
+# Copyright (c) 2012-2015 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
@@ -21,6 +21,7 @@ from __future__ import unicode_literals
 
 from django.utils.http import urlquote
 import rdflib
+import six
 
 from wirecloud.commons.utils.translation import replace_trans_index
 
@@ -30,14 +31,8 @@ WIRE_M = rdflib.Namespace("http://wirecloud.conwet.fi.upm.es/ns/mashup#")
 FOAF = rdflib.Namespace('http://xmlns.com/foaf/0.1/')
 RDF = rdflib.Namespace('http://www.w3.org/1999/02/22-rdf-syntax-ns#')
 RDFS = rdflib.Namespace('http://www.w3.org/2000/01/rdf-schema#')
-MSM = rdflib.Namespace('http://cms-wg.sti2.org/ns/minimal-service-model#')
-OWL = rdflib.Namespace('http://www.w3.org/2002/07/owl#')
 DCTERMS = rdflib.Namespace('http://purl.org/dc/terms/')
 USDL = rdflib.Namespace('http://www.linked-usdl.org/ns/usdl-core#')
-LEGAL = rdflib.Namespace('http://www.linked-usdl.org/ns/usdl-legal#')
-PRICE = rdflib.Namespace('http://www.linked-usdl.org/ns/usdl-pricing#')
-SLA = rdflib.Namespace('http://www.linked-usdl.org/ns/usdl-sla#')
-BLUEPRINT = rdflib.Namespace('http://bizweb.sap.com/TR/blueprint#')
 VCARD = rdflib.Namespace('http://www.w3.org/2006/vcard/ns#')
 XSD = rdflib.Namespace('http://www.w3.org/2001/XMLSchema#')
 CTAG = rdflib.Namespace('http://commontag.org/ns#')
@@ -51,12 +46,12 @@ DOAP = rdflib.Namespace('http://usefulinc.com/ns/doap#')
 def add_translated_nodes(graph, parent_node, namespace, element_name, value, usage, template_info):
 
     used_translation_vars = []
-    for translation_var_name, translation_var_usage in template_info['translation_index_usage'].iteritems():
+    for translation_var_name, translation_var_usage in six.iteritems(template_info['translation_index_usage']):
         if usage in translation_var_usage:
             used_translation_vars.append(translation_var_name)
 
     if len(used_translation_vars) > 0:
-        for lang, catalogue in template_info['translations'].iteritems():
+        for lang, catalogue in six.iteritems(template_info['translations']):
             msg = value
             for translation_var_name in used_translation_vars:
                 msg = replace_trans_index(translation_var_name, catalogue[translation_var_name], msg)
@@ -66,64 +61,101 @@ def add_translated_nodes(graph, parent_node, namespace, element_name, value, usa
         graph.add((parent_node, namespace[element_name], rdflib.Literal(value)))
 
 
-def write_wiring_views_graph(graph, wiring, template_info):
+def write_wiring_components_graph(graph, behaviour, components, type):
 
-    for view in template_info['wiring']['views']:
+    for key, component in six.iteritems(components[type]):
+        component_view = rdflib.BNode()
+        graph.add((component_view, rdflib.RDF.type, WIRE_M['ComponentView']))
+        graph.add((component_view, WIRE['type'], rdflib.Literal(type)))
+        graph.add((component_view, WIRE['id'], rdflib.Literal(str(key))))
+
+        if component.get('collapsed', False):
+            graph.add((component_view, WIRE_M['collapsed'], rdflib.Literal('true')))
+
+        if 'position' in component:
+            write_position_graph(graph, component_view, component['position'])
+
+        if 'endpoints' in component:
+            if 'source' in component['endpoints']:
+                for index, source in enumerate(component['endpoints']['source']):
+                    source_element = rdflib.BNode()
+                    graph.add((source_element, rdflib.RDF.type, WIRE_M['Source']))
+                    graph.add((component_view, WIRE_M['hasSource'], source_element))
+                    graph.add((source_element, RDFS['label'], rdflib.Literal(source)))
+                    graph.add((source_element, WIRE['index'], rdflib.Literal(str(index))))
+
+            if 'target' in component['endpoints']:
+                for index, target in enumerate(component['endpoints']['target']):
+                    target_element = rdflib.BNode()
+                    graph.add((target_element, rdflib.RDF.type, WIRE_M['Target']))
+                    graph.add((component_view, WIRE_M['hasTarget'], target_element))
+                    graph.add((target_element, RDFS['label'], rdflib.Literal(target)))
+                    graph.add((target_element, WIRE['index'], rdflib.Literal(str(index))))
+
+        graph.add((behaviour, WIRE_M['hasComponentView'], component_view))
+
+
+def write_endpoint_graph(graph, node, endpointView, relationName='hasEndpoint'):
+    component_type, component_id, endpoint_name = endpointView.split("/")
+    endpoint = rdflib.BNode()
+
+    graph.add((endpoint, rdflib.RDF.type, WIRE_M['Endpoint']))
+    graph.add((endpoint, WIRE['type'], rdflib.Literal(component_type)))
+    graph.add((endpoint, WIRE_M['id'], rdflib.Literal(str(component_id))))
+    graph.add((endpoint, WIRE_M['endpoint'], rdflib.Literal(endpoint_name)))
+
+    graph.add((node, WIRE_M[relationName], endpoint))
+
+
+def write_position_graph(graph, node, positionView, relationName='hasPosition'):
+    position = rdflib.BNode()
+
+    graph.add((position, rdflib.RDF.type, WIRE_M['Position']))
+    graph.add((position, WIRE_M['x'], rdflib.Literal(str(positionView['x']))))
+    graph.add((position, WIRE_M['y'], rdflib.Literal(str(positionView['y']))))
+
+    graph.add((node, WIRE_M[relationName], position))
+
+
+def write_wiring_connections_graph(graph, behaviour, connections):
+
+    for connection in connections:
+        connection_view = rdflib.BNode()
+        graph.add((connection_view, rdflib.RDF.type, WIRE_M['ConnectionView']))
+
+        # write source endpoint
+        write_endpoint_graph(graph, connection_view, connection['sourcename'], relationName="hasSourceEndpoint")
+
+        if connection.get('sourcehandle', 'auto') != 'auto':
+            write_position_graph(graph, connection_view, connection['sourcehandle'], relationName="hasSourceHandlePosition")
+
+        # write target endpoint
+        write_endpoint_graph(graph, connection_view, connection['targetname'], relationName="hasTargetEndpoint")
+
+        if connection.get('targethandle', 'auto') != 'auto':
+            write_position_graph(graph, connection_view, connection['targethandle'], relationName="hasTargetHandlePosition")
+
+        graph.add((behaviour, WIRE_M['hasConnectionView'], connection_view))
+
+
+def write_wiring_visualdescription_graph(graph, wiring, template_info):
+
+    write_wiring_components_graph(graph, wiring, template_info['wiring']['visualdescription']['components'], 'widget')
+    write_wiring_components_graph(graph, wiring, template_info['wiring']['visualdescription']['components'], 'operator')
+    write_wiring_connections_graph(graph, wiring, template_info['wiring']['visualdescription']['connections'])
+
+    for index, behaviour in enumerate(template_info['wiring']['visualdescription']['behaviours']):
         wiring_view = rdflib.BNode()
-        graph.add((wiring_view, rdflib.RDF.type, WIRE_M['WiringView']))
-        graph.add((wiring, WIRE_M['hasWiringView'], wiring_view))
-        graph.add((wiring_view, RDFS['label'], rdflib.Literal(view['label'])))
+        graph.add((wiring_view, rdflib.RDF.type, WIRE_M['Behaviour']))
+        graph.add((wiring_view, WIRE['index'], rdflib.Literal(str(index))))
+        graph.add((wiring_view, RDFS['label'], rdflib.Literal(behaviour['title'])))
+        graph.add((wiring_view, DCTERMS['description'], rdflib.Literal(behaviour['description'])))
 
-        for key, widget in view['iwidgets'].iteritems():
-            widget_view = rdflib.BNode()
-            graph.add((widget_view, rdflib.RDF.type, WIRE_M['View']))
-            graph.add((widget_view, WIRE['type'], rdflib.Literal('widget')))
-            graph.add((wiring_view, WIRE_M['hasView'], widget_view))
-            graph.add((widget_view, WIRE['id'], rdflib.Literal(str(key))))
-            position = rdflib.BNode()
-            graph.add((position, rdflib.RDF.type, WIRE_M['Position']))
-            graph.add((widget_view, WIRE_M['hasPosition'], position))
-            graph.add((position, WIRE_M['x'], rdflib.Literal(str(widget['position']['posX']))))
-            graph.add((position, WIRE_M['y'], rdflib.Literal(str(widget['position']['posY']))))
-            for index, source in enumerate(widget['endPointsInOuts']['sources']):
-                source_element = rdflib.BNode()
-                graph.add((source_element, rdflib.RDF.type, WIRE_M['Source']))
-                graph.add((widget_view, WIRE_M['hasSource'], source_element))
-                graph.add((source_element, RDFS['label'], rdflib.Literal(source)))
-                graph.add((source_element, WIRE['index'], rdflib.Literal(str(index))))
+        write_wiring_components_graph(graph, wiring_view, behaviour['components'], 'widget')
+        write_wiring_components_graph(graph, wiring_view, behaviour['components'], 'operator')
+        write_wiring_connections_graph(graph, wiring_view, behaviour['connections'])
 
-            for index, target in enumerate(widget['endPointsInOuts']['targets']):
-                target_element = rdflib.BNode()
-                graph.add((target_element, rdflib.RDF.type, WIRE_M['Target']))
-                graph.add((widget_view, WIRE_M['hasTarget'], target_element))
-                graph.add((target_element, RDFS['label'], rdflib.Literal(target)))
-                graph.add((target_element, WIRE['index'], rdflib.Literal(str(index))))
-
-        for key, operator in view['operators'].iteritems():
-            operator_view = rdflib.BNode()
-            graph.add((operator_view, rdflib.RDF.type, WIRE_M['View']))
-            graph.add((operator_view, WIRE['type'], rdflib.Literal('operator')))
-            graph.add((wiring_view, WIRE_M['hasView'], operator_view))
-            graph.add((operator_view, WIRE['id'], rdflib.Literal(str(key))))
-            position = rdflib.BNode()
-            graph.add((position, rdflib.RDF.type, WIRE_M['Position']))
-            graph.add((operator_view, WIRE_M['hasPosition'], position))
-            graph.add((position, WIRE_M['x'], rdflib.Literal(str(operator['position']['posX']))))
-            graph.add((position, WIRE_M['y'], rdflib.Literal(str(operator['position']['posY']))))
-
-            for index, source in enumerate(operator['endPointsInOuts']['sources']):
-                source_element = rdflib.BNode()
-                graph.add((source_element, rdflib.RDF.type, WIRE_M['Source']))
-                graph.add((operator_view, WIRE_M['hasSource'], source_element))
-                graph.add((source_element, RDFS['label'], rdflib.Literal(source)))
-                graph.add((source_element, WIRE['index'], rdflib.Literal(str(index))))
-
-            for index, target in enumerate(operator['endPointsInOuts']['targets']):
-                target_element = rdflib.BNode()
-                graph.add((target_element, rdflib.RDF.type, WIRE_M['Target']))
-                graph.add((operator_view, WIRE_M['hasTarget'], target_element))
-                graph.add((target_element, RDFS['label'], rdflib.Literal(target)))
-                graph.add((target_element, WIRE['index'], rdflib.Literal(str(index))))
+        graph.add((wiring, WIRE_M['hasBehaviour'], wiring_view))
 
 
 def write_mashup_params(graph, resource_uri, template_info):
@@ -135,8 +167,38 @@ def write_mashup_params(graph, resource_uri, template_info):
             graph.add((param_node, WIRE['index'], rdflib.Literal(str(param_index))))
             graph.add((param_node, RDFS['label'], rdflib.Literal(param['label'])))
             graph.add((param_node, WIRE['type'], rdflib.Literal(param['type'])))
+            graph.add((param_node, DCTERMS['description'], rdflib.Literal(param['description'])))
+
+            if param.get('readonly', False) is True:
+                graph.add((param_node, WIRE['readonly'], rdflib.Literal('true')))
+
+            if param.get('default') not in (None, ''):
+                graph.add((param_node, WIRE['default'], rdflib.Literal(param.get('default'))))
+
+            if param.get('value'):
+                graph.add((param_node, WIRE['value'], rdflib.Literal(param['value'])))
+
+            if param.get('required', True) is False:
+                graph.add((param_node, WIRE['required'], rdflib.Literal('false')))
+
             graph.add((resource_uri, WIRE_M['hasMashupParam'], param_node))
 
+
+def write_mashup_embedded_resources(graph, resource_uri, template_info):
+
+    if len(template_info['embedded']) > 0:
+        for resource in template_info['embedded']:
+            resource_node = rdflib.URIRef(resource['src'])
+            graph.add((resource_uri, WIRE_M['hasEmbeddedResource'], resource_node))
+
+            provider_node = rdflib.BNode()
+            graph.add((provider_node, rdflib.RDF.type, GR['BussisnessEntity']))
+            graph.add((provider_node, FOAF['name'], rdflib.Literal(resource['vendor'])))
+            graph.add((resource_node, USDL['hasProvider'], provider_node))
+            graph.add((resource_node, RDFS['label'], rdflib.Literal(resource['name'])))
+            graph.add((resource_node, USDL['versionInfo'], rdflib.Literal(resource['version'])))
+
+
 def write_mashup_resources_graph(graph, resource_uri, template_info):
 
     # Tabs & resources
@@ -145,6 +207,9 @@ def write_mashup_resources_graph(graph, resource_uri, template_info):
         graph.add((tab_element, rdflib.RDF.type, WIRE_M['Tab']))
         graph.add((resource_uri, WIRE_M['hasTab'], tab_element))
         graph.add((tab_element, DCTERMS['title'], rdflib.Literal(tab['name'])))
+        tab_title = tab.get('title', '')
+        if tab_title not in (None, ""):
+            graph.add((tab_element, WIRE['displayName'], rdflib.Literal(tab['title'])))
         graph.add((tab_element, WIRE['index'], rdflib.Literal(str(tab_index))))
 
         for preference_name in tab['preferences']:
@@ -189,47 +254,58 @@ def write_mashup_resources_graph(graph, resource_uri, template_info):
             graph.add((rend, WIRE_M['minimized'], rdflib.Literal(str(iwidget['rendering']['minimized']))))
 
             # iWidget preferences
-            for pref_name, pref in iwidget.get('preferences', {}).iteritems():
+            for pref_name, pref in six.iteritems(iwidget.get('preferences', {})):
                 element = rdflib.BNode()
                 graph.add((element, rdflib.RDF.type, WIRE_M['iWidgetPreference']))
                 graph.add((resource, WIRE_M['hasiWidgetPreference'], element))
                 graph.add((element, DCTERMS['title'], rdflib.Literal(pref_name)))
-                graph.add((element, WIRE['value'], rdflib.Literal(pref['value'])))
+                if pref.get('value', None) is not None:
+                    graph.add((element, WIRE['value'], rdflib.Literal(pref['value'])))
                 if pref.get('readonly', False):
                     graph.add((element, WIRE_M['readonly'], rdflib.Literal('true')))
                 if pref.get('hidden', False):
                     graph.add((element, WIRE_M['hidden'], rdflib.Literal('true')))
 
-            for prop_name, prop in iwidget.get('properties', ()).iteritems():
+            for prop_name, prop in six.iteritems(iwidget.get('properties', ())):
                 element = rdflib.BNode()
                 graph.add((element, rdflib.RDF.type, WIRE_M['iWidgetProperty']))
                 graph.add((resource, WIRE_M['hasiWidgetProperty'], element))
                 graph.add((element, DCTERMS['title'], rdflib.Literal(prop_name)))
-                graph.add((element, WIRE['value'], rdflib.Literal(prop['value'])))
+                if prop.get('value', None) is not None:
+                    graph.add((element, WIRE['value'], rdflib.Literal(prop['value'])))
                 if prop.get('readonly', False):
                     graph.add((element, WIRE_M['readonly'], rdflib.Literal('true')))
 
 
 def write_mashup_wiring_graph(graph, wiring, template_info):
 
-    for id_, operator in template_info['wiring']['operators'].iteritems():
+    wiring_version = template_info['wiring'].get('version', '1.0')
+    if wiring_version != '2.0':
+        raise ValueError('This writer cannot serialize wiring status version %s' % wiring_version)
+
+    graph.add((wiring, USDL['versionInfo'], rdflib.Literal('2.0')))
+
+    # Serialize operators
+    for id_, operator in six.iteritems(template_info['wiring']['operators']):
         op = rdflib.BNode()
         graph.add((op, rdflib.RDF.type, WIRE_M['iOperator']))
         graph.add((wiring, WIRE_M['hasiOperator'], op))
         graph.add((op, DCTERMS['title'], rdflib.Literal(operator['name'])))
         graph.add((op, WIRE_M['iOperatorId'], rdflib.Literal(str(id_))))
 
-        for pref_name, pref in operator['preferences'].iteritems():
+        for pref_name, pref in six.iteritems(operator['preferences']):
             element = rdflib.BNode()
             graph.add((element, rdflib.RDF.type, WIRE_M['iOperatorPreference']))
             graph.add((op, WIRE_M['hasiOperatorPreference'], element))
             graph.add((element, DCTERMS['title'], rdflib.Literal(pref_name)))
-            graph.add((element, WIRE['value'], rdflib.Literal(pref['value'])))
+            if pref.get('value', None) is not None:
+                graph.add((element, WIRE['value'], rdflib.Literal(pref['value'])))
             if pref.get('readonly', False):
                 graph.add((element, WIRE_M['readonly'], rdflib.Literal('true')))
             if pref.get('hidden', False):
                 graph.add((element, WIRE_M['hidden'], rdflib.Literal('true')))
 
+    # Serialize connections
     for connection in template_info['wiring']['connections']:
         element = rdflib.BNode()
         graph.add((element, rdflib.RDF.type, WIRE_M['Connection']))
@@ -252,7 +328,28 @@ def write_mashup_wiring_graph(graph, wiring, template_info):
         graph.add((target, WIRE_M['targetId'], rdflib.Literal(str(connection['target']['id']))))
         graph.add((target, WIRE_M['endpoint'], rdflib.Literal(connection['target']['endpoint'])))
 
-    write_wiring_views_graph(graph, wiring, template_info)
+    write_wiring_visualdescription_graph(graph, wiring, template_info)
+
+
+def write_contents_node(graph, resource_uri, contents_info, alternative=True):
+
+    contents_node = rdflib.URIRef(contents_info.get('src'))
+    graph.add((contents_node, rdflib.RDF.type, USDL['Resource']))
+    graph.add((resource_uri, USDL['utilizedResource'], contents_node))
+
+    if contents_info['contenttype'] != 'text/html' or contents_info['charset'] != 'utf-8':
+        contenttype = contents_info.get('contenttype', 'text/html') + '; charset=' + contents_info.get('charset', 'utf-8').upper()
+        graph.add((contents_node, DCTERMS['format'], rdflib.Literal(contenttype)))
+
+    if alternative is False:
+        if contents_info['cacheable'] is False:
+            graph.add((contents_node, WIRE['codeCacheable'], rdflib.Literal('false')))
+
+        if contents_info['useplatformstyle']:
+            graph.add((contents_node, WIRE['usePlatformStyle'], rdflib.Literal('true')))
+
+    else:
+        graph.add((contents_node, WIRE['contentsScope'], rdflib.Literal(str(contents_info['scope']))))
 
 
 def build_rdf_graph(template_info):
@@ -286,19 +383,48 @@ def build_rdf_graph(template_info):
     graph.add((provider, FOAF['name'], rdflib.Literal(template_info.get('vendor'))))
     graph.add((resource_uri, USDL['versionInfo'], rdflib.Literal(template_info.get('version'))))
     graph.add((resource_uri, DCTERMS['title'], rdflib.Literal(template_info.get('name'))))
-    add_translated_nodes(graph, resource_uri, DCTERMS, 'description', template_info.get('description'), {'type': 'resource', 'field': 'description'}, template_info)
+    add_translated_nodes(graph, resource_uri, DCTERMS, 'abstract', template_info.get('description'), {'type': 'resource', 'field': 'description'}, template_info)
 
-    authors = rdflib.BNode()
-    graph.add((authors, rdflib.RDF.type, FOAF['Person']))
-    graph.add((resource_uri, DCTERMS['creator'], authors))
-    graph.add((authors, FOAF['name'], rdflib.Literal(template_info.get('authors'))))
+    longdescription = template_info.get('longdescription', None)
+    if longdescription not in (None, ''):
+        graph.add((resource_uri, DCTERMS['description'], rdflib.URIRef(longdescription)))
+
+    for index, author in enumerate(template_info.get('authors', ())):
+        author_node = rdflib.BNode()
+        graph.add((resource_uri, DCTERMS['creator'], author_node))
+        graph.add((author_node, rdflib.RDF.type, FOAF['Person']))
+        graph.add((author_node, WIRE['index'], rdflib.Literal(str(index))))
+        graph.add((author_node, FOAF['name'], rdflib.Literal(author['name'])))
+        if 'email' in author:
+            graph.add((author_node, FOAF['mbox'], rdflib.Literal(author['email'])))
+        if 'url' in author:
+            graph.add((author_node, FOAF['homepage'], rdflib.Literal(author['url'])))
+
+    for index, contributor in enumerate(template_info.get('contributors', ())):
+        contributor_node = rdflib.BNode()
+        graph.add((resource_uri, DCTERMS['contributor'], contributor_node))
+        graph.add((contributor_node, rdflib.RDF.type, FOAF['Person']))
+        graph.add((contributor_node, WIRE['index'], rdflib.Literal(str(index))))
+        graph.add((contributor_node, FOAF['name'], rdflib.Literal(contributor['name'])))
+        if 'email' in contributor:
+            graph.add((contributor_node, FOAF['mbox'], rdflib.Literal(contributor['email'])))
+        if 'url' in contributor:
+            graph.add((contributor_node, FOAF['homepage'], rdflib.Literal(contributor['url'])))
 
     graph.add((resource_uri, WIRE['hasImageUri'], rdflib.URIRef(template_info.get('image', ''))))
 
+    graph.add((resource_uri, WIRE['hasChangeLog'], rdflib.URIRef(template_info.get('changelog', ''))))
+
+    homepage = template_info.get('homepage')
+    if homepage not in (None, ''):
+        graph.add((resource_uri, FOAF['homepage'], rdflib.URIRef(homepage)))
+
     if template_info.get('doc'):
         graph.add((resource_uri, FOAF['page'], rdflib.URIRef(template_info.get('doc'))))
 
-    add_translated_nodes(graph, resource_uri, WIRE, 'displayName', template_info.get('title', ''), {'type': 'resource', 'field': 'title'}, template_info)
+    display_name = template_info.get('title', '')
+    if display_name not in (None, ''):
+        add_translated_nodes(graph, resource_uri, WIRE, 'displayName', display_name, {'type': 'resource', 'field': 'title'}, template_info)
 
     license_url_text = template_info.get('licenseurl', None)
     if license_url_text not in (None, ''):
@@ -309,10 +435,16 @@ def build_rdf_graph(template_info):
             graph.add((license, rdflib.RDF.type, DCTERMS['LicenseDocument']))
             graph.add((license, RDFS['label'], rdflib.Literal(license_text)))
 
-    addr = rdflib.BNode()
-    graph.add((addr, rdflib.RDF.type, VCARD['Work']))
-    graph.add((resource_uri, VCARD['addr'], addr))
-    graph.add((addr, VCARD['email'], rdflib.Literal(template_info.get('email'))))
+    issuetracker = template_info.get('issuetracker', None)
+    if issuetracker not in (None, ''):
+        graph.add((resource_uri, DOAP['bug-database'], rdflib.URIRef(issuetracker)))
+
+    contact_email = template_info.get('email', None)
+    if contact_email not in (None, ''):
+        addr = rdflib.BNode()
+        graph.add((addr, rdflib.RDF.type, VCARD['Work']))
+        graph.add((resource_uri, VCARD['addr'], addr))
+        graph.add((addr, VCARD['email'], rdflib.Literal(contact_email)))
 
     # Requirements
     for requirement in template_info['requirements']:
@@ -327,6 +459,7 @@ def build_rdf_graph(template_info):
     # Params
     if template_info['type'] == 'mashup':
         write_mashup_params(graph, resource_uri, template_info)
+        write_mashup_embedded_resources(graph, resource_uri, template_info)
 
     # Create wiring
     wiring = rdflib.BNode()
@@ -343,8 +476,8 @@ def build_rdf_graph(template_info):
         graph.add((wiring, WIRE['hasOutputEndpoint'], output_node))
         graph.add((output_node, WIRE['index'], rdflib.Literal(str(output_index))))
         graph.add((output_node, DCTERMS['title'], rdflib.Literal(output_endpoint.get('name'))))
-        add_translated_nodes(graph, output_node, RDFS, 'label', output_endpoint.get('label'), {'type': 'vdef', 'variable': output_endpoint.get('name'), 'field': 'label'}, template_info)
-        add_translated_nodes(graph, output_node, DCTERMS, 'description', output_endpoint.get('description'), {'type': 'vdef', 'variable': output_endpoint.get('name'), 'field': 'description'}, template_info)
+        add_translated_nodes(graph, output_node, RDFS, 'label', output_endpoint.get('label'), {'type': 'outputendpoint', 'variable': output_endpoint.get('name'), 'field': 'label'}, template_info)
+        add_translated_nodes(graph, output_node, DCTERMS, 'description', output_endpoint.get('description'), {'type': 'outputendpoint', 'variable': output_endpoint.get('name'), 'field': 'description'}, template_info)
         graph.add((output_node, WIRE['type'], rdflib.Literal(output_endpoint.get('type'))))
         graph.add((output_node, WIRE['friendcode'], rdflib.Literal(output_endpoint.get('friendcode'))))
 
@@ -355,11 +488,11 @@ def build_rdf_graph(template_info):
         graph.add((wiring, WIRE['hasInputEndpoint'], input_node))
         graph.add((input_node, WIRE['index'], rdflib.Literal(str(input_index))))
         graph.add((input_node, DCTERMS['title'], rdflib.Literal(input_endpoint.get('name'))))
-        add_translated_nodes(graph, input_node, RDFS, 'label', input_endpoint.get('label'), {'type': 'vdef', 'variable': input_endpoint.get('name'), 'field': 'label'}, template_info)
-        add_translated_nodes(graph, input_node, DCTERMS, 'description', input_endpoint.get('description'), {'type': 'vdef', 'variable': input_endpoint.get('name'), 'field': 'description'}, template_info)
+        add_translated_nodes(graph, input_node, RDFS, 'label', input_endpoint.get('label'), {'type': 'inputendpoint', 'variable': input_endpoint.get('name'), 'field': 'label'}, template_info)
+        add_translated_nodes(graph, input_node, DCTERMS, 'description', input_endpoint.get('description'), {'type': 'inputendpoint', 'variable': input_endpoint.get('name'), 'field': 'description'}, template_info)
         graph.add((input_node, WIRE['type'], rdflib.Literal(input_endpoint.get('type'))))
         graph.add((input_node, WIRE['friendcode'], rdflib.Literal(input_endpoint.get('friendcode'))))
-        add_translated_nodes(graph, input_node, WIRE, 'inputActionLabel', input_endpoint.get('actionlabel'), {'type': 'vdef', 'variable': input_endpoint.get('name'), 'field': 'actionlabel'}, template_info)
+        add_translated_nodes(graph, input_node, WIRE, 'inputActionLabel', input_endpoint.get('actionlabel'), {'type': 'inputendpoint', 'variable': input_endpoint.get('name'), 'field': 'actionlabel'}, template_info)
 
     if template_info['type'] == 'mashup':
         write_mashup_wiring_graph(graph, wiring, template_info)
@@ -369,7 +502,7 @@ def build_rdf_graph(template_info):
 
     if template_info['type'] == 'mashup':
         # Mashup preferences
-        for pref_name, pref_value in template_info['preferences'].iteritems():
+        for pref_name, pref_value in six.iteritems(template_info['preferences']):
             pref = rdflib.BNode()
             graph.add((pref, rdflib.RDF.type, WIRE_M['MashupPreference']))
             graph.add((resource_uri, WIRE_M['hasMashupPreference'], pref))
@@ -399,6 +532,9 @@ def build_rdf_graph(template_info):
             if pref.get('secure'):
                 graph.add((pref_node, WIRE['secure'], rdflib.Literal(pref.get('secure'))))
 
+            if pref.get('required', False) is True:
+                graph.add((pref_node, WIRE['required'], rdflib.Literal('true')))
+
             if pref.get('options'):
                 for option_index, option in enumerate(pref['options']):
                     option_node = rdflib.BNode()
@@ -425,21 +561,16 @@ def build_rdf_graph(template_info):
             if prop.get('secure'):
                 graph.add((prop_node, WIRE['secure'], rdflib.Literal(prop.get('secure'))))
 
+            if prop.get('multiuser', False) is True:
+                graph.add((prop_node, WIRE['multiuser'], rdflib.Literal('true')))
+
     # Code
     if template_info['type'] == 'widget':
-        xhtml_element = rdflib.URIRef(template_info.get('code_url'))
-        graph.add((xhtml_element, rdflib.RDF.type, USDL['Resource']))
-        graph.add((resource_uri, USDL['utilizedResource'], xhtml_element))
 
-        if template_info['code_content_type'] != 'text/html' or template_info['code_charset'] != 'utf-8':
-            content_type = template_info.get('code_content_type', 'text/html') + '; charset=' + template_info.get('code_charset', 'utf-8').upper()
-            graph.add((xhtml_element, DCTERMS['format'], rdflib.Literal(content_type)))
+        write_contents_node(graph, resource_uri, template_info['contents'], alternative=False)
 
-        if template_info['code_cacheable'] is False:
-            graph.add((xhtml_element, WIRE['codeCacheable'], rdflib.Literal('false')))
-
-        if template_info['code_uses_platform_style']:
-            graph.add((xhtml_element, WIRE['usePlatformStyle'], rdflib.Literal('true')))
+        for altcontents in template_info['altcontents']:
+            write_contents_node(graph, resource_uri, altcontents)
 
     elif template_info['type'] == 'operator':
         for index, js_file in enumerate(template_info['js_files']):
@@ -465,4 +596,4 @@ def write_rdf_description(template_info, format='pretty-xml'):
         raise Exception('Unsupported resource type: ' + template_info['type'])
 
     graph = build_rdf_graph(template_info)
-    return graph.serialize(format=format)
+    return graph.serialize(format=format, encoding='utf-8').decode('utf-8')
