@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2013 CoNWeT Lab., Universidad Politécnica de Madrid
+# Copyright (c) 2013-2016 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
@@ -17,26 +17,29 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with Wirecloud.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import unicode_literals
+
 import os
 import sys
 
-from django.core.management import LaxOptionParser
-
 import wirecloud.platform
 
 
 class CommandLineUtility(object):
 
-    def __init__(self, commands, argv=None):
+    def __init__(self, commands, prog_name=None):
 
         self.commands = commands
-        self.argv = argv or sys.argv[:]
-        self.prog_name = os.path.basename(self.argv[0])
+        if prog_name is None:
+            self.prog_name = os.path.basename(sys.argv[0])
+        else:
+            self.prog_name = prog_name
 
     def main_help_text(self, commands_only=False):
         """
         Returns the script's main help text, as a string.
         """
+
         if commands_only:
             usage = sorted(self.commands.keys())
         else:
@@ -65,61 +68,87 @@ class CommandLineUtility(object):
         else:
             return None
 
-    def execute(self):
+    def execute(self, argv=None, stdout=None, stderr=None):
 
-        parser = LaxOptionParser(usage="%prog subcommand [options] [args]",
-            version=wirecloud.platform.__version__,
-            option_list=())
+        from django.core.management.base import CommandParser
+
+        if argv is None:
+            argv = sys.argv
+
+        if stdout is None:
+            stdout = sys.stdout
+
+        if stderr is None:
+            stderr = sys.stderr
+
+        parser = CommandParser(
+            None,
+            usage="%(prog)s subcommand [options] [args]",
+            add_help=False
+        )
+        parser.add_argument('--version', action='store_true', help="show program's version number and exit")
+        parser.add_argument('-h', '--help', action='store_true', help="show this help message and exit")
 
         try:
-            options, args = parser.parse_args(self.argv)
+            options, argv = parser.parse_known_args(argv)
         except:
             pass  # Ignore any option errors at this point.
 
-        try:
-            subcommand = self.argv[1]
-        except IndexError:
+        if len(argv) > 1:
+            subcommand = argv[1]
+        else:
             subcommand = 'help'  # Display help if no arguments were given.
 
-        if subcommand == 'help':
-            if len(args) <= 2:
-                parser.print_lax_help()
-                sys.stdout.write(self.main_help_text() + '\n')
-            elif args[2] == '--commands':
-                sys.stdout.write(self.main_help_text(commands_only=True) + '\n')
+        if options.help:
+
+            if subcommand == 'help':
+                parser.print_help(stdout)
+                stdout.write(self.main_help_text() + '\n')
             else:
-                command = self.fetch_command(args[2])
+                command = self.fetch_command(subcommand)
                 if command is not None:
-                    command.print_help(self.prog_name, args[2])
+                    command.print_help(self.prog_name, subcommand, file=stdout)
                 else:
-                    sys.stdout.write(self.unknown_command_text(args[2]) + '\n')
+                    stdout.write(self.unknown_command_text(subcommand) + '\n')
+
+        elif subcommand == 'version' or options.version:
+
+            stdout.write(wirecloud.platform.__version__ + '\n')
+
+        elif subcommand == 'help':
+
+            if len(argv) <= 2:
+                parser.print_help(stdout)
+                stdout.write(self.main_help_text() + '\n')
+            elif argv[2] == '--commands':
+                stdout.write(self.main_help_text(commands_only=True) + '\n')
+            else:
+                command = self.fetch_command(argv[2])
+                if command is not None:
+                    command.print_help(self.prog_name, argv[2], file=stdout)
+                else:
+                    stdout.write(self.unknown_command_text(argv[2]) + '\n')
 
-        elif subcommand == 'version':
-            sys.stdout.write(parser.get_version() + '\n')
-        elif '--version' in self.argv[1:]:
-            # LaxOptionParser already takes care of printing the version.
-            pass
-        elif '--help' in self.argv[1:] or '-h' in self.argv[1:]:
-            parser.print_lax_help()
-            sys.stdout.write(self.main_help_text() + '\n')
         else:
             command = self.fetch_command(subcommand)
             if command is not None:
-                command.run_from_argv(self.argv)
+                command.run_from_argv(argv, stdout=stdout, stderr=stderr)
             else:
-                sys.stdout.write(self.unknown_command_text(subcommand) + '\n')
+                stdout.write(self.unknown_command_text(subcommand) + '\n')
 
 
 def execute_from_command_line():
     from wirecloud.commons.commands.convert import ConvertCommand
     from wirecloud.commons.commands.startproject import StartprojectCommand
-    from wirecloud.fiware.commands.passintegrationtests import IntegrationTestsCommand
     utility = CommandLineUtility({
         "convert": ConvertCommand(),
-        "startproject": StartprojectCommand(),
-        "passintegrationtests": IntegrationTestsCommand(),
+        "startproject": StartprojectCommand()
     })
-    utility.execute()
+    try:
+        utility.execute()
+    except:
+        sys.exit(1)
+
 
 if __name__ == "__main__":
     execute_from_command_line()
