@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2012-2013 CoNWeT Lab., Universidad Politécnica de Madrid
+# Copyright (c) 2012-2015 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
@@ -17,16 +17,17 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with Wirecloud.  If not, see <http://www.gnu.org/licenses/>.
 
-import requests
-from requests.auth import HTTPBasicAuth
-from urllib2 import URLError, HTTPError
-from urlparse import urljoin, urlparse
-from lxml import etree
+import concurrent.futures
 
 from django.core.cache import cache
 from django.utils.http import urlquote, urlquote_plus
+import requests
+from requests.auth import HTTPBasicAuth
+from six.moves.urllib.error import URLError, HTTPError
+from six.moves.urllib.parse import urljoin, urlparse, urlunparse
+from lxml import etree
 
-from wirecloud.commons.utils.http import parse_mime_type
+from wirecloud.commons.utils.mimeparser import parse_mime_type
 from wirecloud.fiware.marketAdaptor.usdlParser import USDLParseException, USDLParser
 from wirecloud.fiware.storeclient import StoreClient
 
@@ -74,23 +75,111 @@ def parse_resource_info(offering_resource):
         'name': offering_resource['name'],
         'description': offering_resource['description'],
     }
+
+    if 'metadata' in offering_resource and len(offering_resource['metadata']) > 0:
+        resource_info['metadata'] = offering_resource['metadata']
+
     if 'link' in offering_resource:
         resource_info['url'] = offering_resource['link']
 
     if offering_resource['content_type'] in ('application/x-widget+mashable-application-component', 'application/x-operator+mashable-application-component', 'application/x-mashup+mashable-application-component'):
-        if 'link' in offering_resource:
+        if 'metadata' in resource_info:
+            resource_info['id'] = '/'.join((resource_info['metadata']['vendor'], resource_info['metadata']['name'], resource_info['metadata']['version']))
+        elif 'link' in offering_resource:
             resource_info['id'] = offering_resource['link'].rsplit('__', 1)[1].rsplit('.', 1)[0].replace('_', '/')
 
     return resource_info
 
 
+def _parse_offering_store_info(ser, store_client, offering_id, token):
+    try:
+        offering_info = store_client.get_offering_info(offering_id, token)
+    except:
+        return
+
+    offering_type = 'other'
+
+    if 'resources' not in offering_info:
+        offering_info['resources'] = []
+
+    if len(offering_info['resources']) == 1:
+
+        offering_resource = offering_info['resources'][0]
+
+        if offering_resource['content_type'] == 'application/x-widget+mashable-application-component':
+            offering_type = 'widget'
+        elif offering_resource['content_type'] == 'application/x-operator+mashable-application-component':
+            offering_type = 'operator'
+        elif offering_resource['content_type'] == 'application/x-mashup+mashable-application-component':
+            offering_type = 'mashup'
+
+        ser['resources'] = [parse_resource_info(offering_resource)]
+    else:
+
+        info_offering_resources = []
+        for offering_resource in offering_info['resources']:
+            resource_info = parse_resource_info(offering_resource)
+
+            if resource_info['content_type'] in ('application/x-widget+mashable-application-component', 'application/x-operator+mashable-application-component', 'application/x-mashup+mashable-application-component'):
+                offering_type = 'pack'
+
+            info_offering_resources.append(resource_info)
+
+        ser['resources'] = info_offering_resources
+
+    ser['type'] = offering_type
+    ser['uriImage'] = urljoin(store_client._url, offering_info['image_url'])
+    ser['publicationdate'] = offering_info['publication_date']
+    ser['open'] = offering_info['open']
+    ser['state'] = offering_info['state']
+    ser['rating'] = offering_info['rating']
+    ser['vendor'] = offering_info['owner_organization']
+    ser['version'] = offering_info['version']
+
+
+def update_offering_info(ser, adaptor, store, name, url, options):
+
+    ser['store'] = store
+    ser['id'] = name
+
+    if ser['uriTemplate'] == '':
+        ser['uriTemplate'] = url
+
+    ser['usdl_url'] = url
+    ser['type'] = 'unknown'
+    ser['resources'] = []
+
+    try:
+
+        offering_parsed_url = urlparse(url)
+        offering_id = offering_parsed_url.path.rsplit('/', 1)[1].replace('__', '/')
+
+        store_client = adaptor.get_store(store)
+        store_token_key = store + '/token'
+        if store_token_key in options:
+            token = options[store_token_key]
+        else:
+            token = options['idm_token']
+
+        _parse_offering_store_info(ser, store_client, offering_id, token)
+
+    except:
+        pass
+
+    return ser
+
+
 class MarketAdaptor(object):
 
     _marketplace_uri = None
     _stores = {}
 
     def __init__(self, marketplace_uri, user='demo1234', passwd='demo1234'):
-        self._marketplace_uri = marketplace_uri
+        url = urlparse(marketplace_uri)
+        if not bool(url.netloc and url.scheme):
+            raise ValueError("Your must provide an absolute Marketplace URL")
+
+        self._marketplace_uri = urlunparse((url.scheme, url.netloc, url.path.rstrip('/') + '/', '', '', ''))
         self._user = user
         self._passwd = passwd
 
@@ -106,87 +195,27 @@ class MarketAdaptor(object):
 
         return result
 
-    def _parse_offering(self, name, url, parsed_usdl, store, options):
+    def _parse_offering(self, name, url, parsed_usdl, store, options, executor):
 
-        offerings = []
+        jobs = []
 
         if isinstance(parsed_usdl, dict):
             parsed_usdl = [parsed_usdl]
 
         for ser in parsed_usdl:
 
-            ser['store'] = store
-            ser['marketName'] = name
+            jobs.append(executor.submit(update_offering_info, ser, self, store, name, url, options))
 
-            if ser['uriTemplate'] == '':
-                ser['uriTemplate'] = url
-
-            ser['usdl_url'] = url
-            ser['type'] = 'unknown'
-            ser['resources'] = []
-
-            try:
-
-                offering_parsed_url = urlparse(url)
-                offering_id = offering_parsed_url.path.rsplit('/', 1)[1].replace('__', '/')
-
-                store_client = self._stores[store]
-                store_token_key = store + '/token'
-                if store_token_key in options:
-                    token = options[store_token_key]
-                else:
-                    token = options['idm_token']
-
-                offering_info = store_client.get_offering_info(offering_id, token)
-                offering_type = 'other'
-                if len(offering_info['resources']) == 1:
-
-                    offering_resource = offering_info['resources'][0]
-
-                    if offering_resource['content_type'] == 'application/x-widget+mashable-application-component':
-                        offering_type = 'widget'
-                    elif offering_resource['content_type'] == 'application/x-operator+mashable-application-component':
-                        offering_type = 'operator'
-                    elif offering_resource['content_type'] == 'application/x-mashup+mashable-application-component':
-                        offering_type = 'mashup'
-
-                    ser['resources'] = [parse_resource_info(offering_resource)]
-                else:
-
-                    info_offering_resources = []
-                    for offering_resource in offering_info['resources']:
-                        resource_info = parse_resource_info(offering_resource)
-
-                        if resource_info['content_type'] in ('application/x-widget+mashable-application-component', 'application/x-operator+mashable-application-component', 'application/x-mashup+mashable-application-component'):
-                            offering_type = 'pack'
-
-                        info_offering_resources.append(resource_info)
-
-                    ser['resources'] = info_offering_resources
-
-                ser['type'] = offering_type
-                ser['uriImage'] = urljoin(store_client._url, offering_info['image_url'])
-                ser['publicationdate'] = offering_info['publication_date']
-                ser['state'] = offering_info['state']
-                ser['rating'] = offering_info['rating']
-
-            except:
-                pass
-
-            offerings.append(ser)
-
-        return offerings
+        return jobs
 
     def get_all_stores(self):
 
         url = urljoin(self._marketplace_uri, "registration/stores/")
-        try:
-            response = requests.get(url, auth=HTTPBasicAuth(self._user, self._passwd))
-        except HTTPError as e:
-            if e.code == 404:
-                return []
+        response = requests.get(url, auth=HTTPBasicAuth(self._user, self._passwd))
 
-        if response.status_code != 200:
+        if response.status_code == 404:
+            return []
+        elif response.status_code != 200:
             raise HTTPError(response.url, response.status_code, response.reason, None, None)
 
         parsed_body = etree.fromstring(response.content)
@@ -201,10 +230,7 @@ class MarketAdaptor(object):
     def get_store_info(self, store):
 
         url = urljoin(self._marketplace_uri, "registration/store/" + urlquote(store))
-        try:
-            response = requests.get(url, auth=HTTPBasicAuth(self._user, self._passwd))
-        except HTTPError as e:
-            raise HTTPError(e.url, e.code, e.msg, None, None)
+        response = requests.get(url, auth=HTTPBasicAuth(self._user, self._passwd))
 
         if response.status_code != 200:
             raise HTTPError(response.url, response.status_code, response.reason, None, None)
@@ -215,45 +241,50 @@ class MarketAdaptor(object):
     def get_all_services_from_store(self, store, **options):
 
         url = urljoin(self._marketplace_uri, "offering/store/" + urlquote(store) + "/offerings")
-        try:
-            response = requests.get(url, auth=HTTPBasicAuth(self._user, self._passwd))
-        except HTTPError as e:
-            raise HTTPError(e.url, e.code, e.msg, None, None)
+        response = requests.get(url, auth=HTTPBasicAuth(self._user, self._passwd))
 
         if response.status_code != 200:
             raise HTTPError(response.url, response.status_code, response.reason, None, None)
 
         parsed_body = etree.fromstring(response.content)
 
-        result = {'resources': []}
+        jobs = []
 
-        for res in parsed_body.xpath(RESOURCE_XPATH):
+        with concurrent.futures.ThreadPoolExecutor(max_workers=5) as executor:
+            for res in parsed_body.xpath(RESOURCE_XPATH):
 
-            url = res.xpath(URL_XPATH)[0].text
+                url = res.xpath(URL_XPATH)[0].text
 
-            try:
-                parsed_usdl = parse_usdl_from_url(url)
-            except:
-                continue
+                try:
+                    parsed_usdl = parse_usdl_from_url(url)
+                except:
+                    continue
 
-            result['resources'] += self._parse_offering(res.get('name'), url, parsed_usdl, store, options)
+                jobs += self._parse_offering(res.get('name'), url, parsed_usdl, store, options, executor)
 
-        return result
+            concurrent.futures.wait(jobs)
+
+        return {'resources': tuple(job.result() for job in jobs)}
 
     def get_offering_info(self, store, id, options):
 
         url = urljoin(self._marketplace_uri, "offering/store/%(store)s/offering/%(offering_id)s" % {"store": urlquote(store), "offering_id": urlquote(id)})
-        try:
-            response = requests.get(url, auth=HTTPBasicAuth(self._user, self._passwd))
-        except HTTPError as e:
-            raise HTTPError(e.url, e.code, e.msg, None, None)
+        response = requests.get(url, auth=HTTPBasicAuth(self._user, self._passwd))
+
+        if response.status_code != 200:
+            raise HTTPError(response.url, response.status_code, response.reason, None, None)
 
         parsed_body = etree.fromstring(response.content)
         url = parsed_body.xpath(URL_XPATH)[0].text
 
         parsed_usdl = parse_usdl_from_url(url)
 
-        return self._parse_offering(id, url, parsed_usdl, store, options)
+        self.get_store(store)
+        with concurrent.futures.ThreadPoolExecutor(max_workers=5) as executor:
+            jobs = self._parse_offering(id, url, parsed_usdl, store, options, executor)
+            concurrent.futures.wait(jobs)
+
+        return tuple(job.result() for job in jobs)
 
     def get_store(self, name):
         if name not in self._stores:
@@ -273,30 +304,28 @@ class MarketAdaptor(object):
     def full_text_search(self, store, search_string, options):
 
         url = urljoin(self._marketplace_uri, "search/offerings/fulltext/" + urlquote_plus(search_string))
-        try:
-            response = requests.get(url, auth=HTTPBasicAuth(self._user, self._passwd))
-        except HTTPError as e:
-            raise HTTPError(e.url, e.code, e.msg, None, None)
+        response = requests.get(url, auth=HTTPBasicAuth(self._user, self._passwd))
 
         if response.status_code != 200:
             raise HTTPError(response.url, response.status_code, response.reason, None, None)
 
         parsed_body = etree.fromstring(response.content)
 
-        result = {'resources': []}
-        for res in parsed_body.xpath(SEARCH_RESULT_XPATH):
-            service = res.xpath(SEARCH_SERVICE_XPATH)[0]
-            url = service.xpath(URL_XPATH)[0].text
-            service_store = res.xpath(SEARCH_STORE_XPATH)[0].get('name')
+        jobs = []
+        with concurrent.futures.ThreadPoolExecutor(max_workers=5) as executor:
+            for res in parsed_body.xpath(SEARCH_RESULT_XPATH):
+                service = res.xpath(SEARCH_SERVICE_XPATH)[0]
+                url = service.xpath(URL_XPATH)[0].text
+                service_store = res.xpath(SEARCH_STORE_XPATH)[0].get('name')
 
-            if store != '' and store != service_store:
-                continue
+                if store != '' and store != service_store:
+                    continue
 
-            try:
-                parsed_usdl = parse_usdl_from_url(url)
-            except:
-                continue
+                try:
+                    parsed_usdl = parse_usdl_from_url(url)
+                except:
+                    continue
 
-            result['resources'] += self._parse_offering(res.get('name'), url, parsed_usdl, service_store, options)
+                jobs += self._parse_offering(res.get('name'), url, parsed_usdl, service_store, options, executor)
 
-        return result
+        return {'resources': tuple(job.result() for job in jobs)}
