@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2011-2014 CoNWeT Lab., Universidad Politécnica de Madrid
+# Copyright (c) 2011-2017 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
@@ -17,5 +17,10 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with Wirecloud.  If not, see <http://www.gnu.org/licenses/>.
 
-__version_info__ = (0, 6, 5)
+__version_info__ = (1, 1, 5)
 __version__ = '.'.join(map(str, __version_info__))
+__application_mashup_version_info__ = (2, 2)
+__application_mashup_version__ = '.'.join(map(str, __application_mashup_version_info__))
+__git_hash__ = "2a9519358434871a9d2a2e6fed29a7918f6ff6ed"
+__git_dirty__ = True
+__release_date__ = "Unknown"
