@@ -92,16 +92,6 @@ def get_workspace_context_current_values(workspace, user):
     return values
 
 
-def get_workspace_context(workspace, user):
-
-    context = copy.deepcopy(get_workspace_context_definitions())
-    values = get_workspace_context_current_values(workspace, user)
-    for key in values:
-        context[key]['value'] = values[key]
-
-    return context
-
-
 def get_constant_context_values():
     res = {}
 
@@ -115,7 +105,7 @@ def get_constant_context_values():
 def get_context_values(workspace, user):
     cache_key = 'constant_context/' + str(user.id)
     constant_context = cache.get(cache_key)
-    if constant_context == None:
+    if constant_context is None:
         constant_context = get_constant_context_values()
         cache.set(cache_key, constant_context)
 
