@@ -1,5 +1,5 @@
 /*
- *     Copyright (c) 2008-2014 CoNWeT Lab., Universidad Politécnica de Madrid
+ *     Copyright (c) 2008-2016 CoNWeT Lab., Universidad Politécnica de Madrid
  *
  *     This file is part of Wirecloud Platform.
  *
@@ -19,29 +19,32 @@
  *
  */
 
-/*global Wirecloud*/
+/* globals StyledElements, Wirecloud */
 
-(function () {
+
+(function (utils) {
 
     "use strict";
 
     /**
      * @private
      */
-    var onSuccessSavePreferences = function onSuccessSavePreferences() {
-        this.preferences.events['post-commit'].dispatch(this.preferences, this.modifiedValues);
+    var onSuccessSavePreferences = function onSuccessSavePreferences(options) {
+        utils.callCallback(options.onSuccess, this.preferences);
+        this.preferences.dispatchEvent('post-commit', this.modifiedValues);
     };
 
     /**
      * @private
      */
-    var onErrorSavePreferences = function onErrorSavePreferences() {
+    var onErrorSavePreferences = function onErrorSavePreferences(options) {
+        utils.callCallback(options.onFailure, this.preferences);
     };
 
     /**
      * @private
      */
-    var persist = function persist(modifiedValues) {
+    var persist = function persist(modifiedValues, options) {
         var context = {
             preferences: this,
             modifiedValues: modifiedValues
@@ -52,8 +55,8 @@
             contentType: 'application/json',
             requestHeaders: {'Accept': 'application/json'},
             postBody: JSON.stringify(modifiedValues),
-            onSuccess: onSuccessSavePreferences.bind(context),
-            onFailure: onErrorSavePreferences.bind(context)
+            onSuccess: onSuccessSavePreferences.bind(context, options),
+            onFailure: onErrorSavePreferences.bind(context, options)
         });
     };
 
@@ -76,7 +79,7 @@
                 inherit = values[definition.name].inherit;
                 value = values[key].value;
             } else {
-                value = definition.default;
+                value = null;
                 inherit = definition.inheritByDefault;
             }
 
@@ -107,10 +110,14 @@
      *
      * @param {Object} newValues a hash with preferenceName/changes pairs
      */
-    Preferences.prototype.set = function set(newValues) {
+    Preferences.prototype.set = function set(newValues, options) {
         var newEffectiveValues = {};
         var modifiedValues = {};
 
+        if (options == null) {
+            options = {};
+        }
+
         for (var name in newValues) {
             var preference = this.preferences[name];
             var changes = newValues[name];
@@ -121,14 +128,15 @@
 
             if ('value' in changes) {
                 preference.value = changes.value;
+                changes.value = Wirecloud.ui.InputInterfaceFactory.stringify(preference.meta.options.type, preference.value);
             }
 
             modifiedValues[name] = changes;
             newEffectiveValues[name] = preference.getEffectiveValue();
         }
 
-        this.events['pre-commit'].dispatch(this, newEffectiveValues);
-        persist.call(this, modifiedValues);
+        this.dispatchEvent('pre-commit', newEffectiveValues);
+        persist.call(this, modifiedValues, options);
     };
 
     Preferences.prototype._handleParentChanges = function _handleParentChanges(parentPreferences, modifiedValues) {
@@ -143,7 +151,7 @@
         }
 
         if (propagate) {
-            this.events['pre-commit'].dispatch(this, valuesToPropagate);
+            this.dispatchEvent('pre-commit', valuesToPropagate);
         }
     };
 
@@ -152,4 +160,4 @@
 
     Wirecloud.Preferences = Preferences;
 
-})();
+})(Wirecloud.Utils);
