@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2013 CoNWeT Lab., Universidad Politécnica de Madrid
+# Copyright (c) 2013-2015 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
@@ -20,18 +20,20 @@
 import os
 
 from django.db.models.signals import post_delete, post_save
+from django.dispatch import receiver
 
 from wirecloud.catalogue.models import CatalogueResource
 from wirecloud.commons.utils.wgt import WgtFile
 
 
+@receiver(post_save, sender=CatalogueResource)
 def deploy_operators_on_resource_creation(sender, instance, created, raw, **kwargs):
 
     from wirecloud.catalogue import utils as catalogue_utils
     import wirecloud.platform.widget.utils as showcase_utils
 
     resource = instance
-    if not created or raw or not resource.fromWGT or resource.resource_type() != 'operator':
+    if not created or raw or resource.resource_type() != 'operator':
         return
 
     base_dir = catalogue_utils.wgt_deployer.get_base_dir(resource.vendor, resource.short_name, resource.version)
@@ -39,17 +41,18 @@ def deploy_operators_on_resource_creation(sender, instance, created, raw, **kwar
 
     showcase_utils.wgt_deployer.deploy(wgt_file)
 
-post_save.connect(deploy_operators_on_resource_creation, sender=CatalogueResource)
-
 
+@receiver(post_delete, sender=CatalogueResource)
 def undeploy_operators_on_resource_deletion(sender, instance, **kwargs):
 
     import wirecloud.platform.widget.utils as showcase_utils
 
     resource = instance
-    if not resource.fromWGT or resource.resource_type() != 'operator':
+    if resource.resource_type() != 'operator':
         return
 
-    showcase_utils.wgt_deployer.undeploy(resource.vendor, resource.short_name, resource.version)
-
-post_delete.connect(undeploy_operators_on_resource_deletion, sender=CatalogueResource)
+    try:
+        showcase_utils.wgt_deployer.undeploy(resource.vendor, resource.short_name, resource.version)
+    except:
+        # TODO log this error
+        pass  # ignore errors
